@@ -1,28 +1,25 @@
-
-#coding: utf-8
+# -*- coding: utf-8 -*-
 from __future__ import print_function
 # pylint: disable=W0141
 
 import sys
-import re
 
 from pandas.core.base import PandasObject
-from pandas.core.common import adjoin, isnull, notnull
+from pandas.core.common import adjoin, notnull
 from pandas.core.index import Index, MultiIndex, _ensure_index
 from pandas import compat
 from pandas.compat import(StringIO, lzip, range, map, zip, reduce, u,
                           OrderedDict)
 from pandas.util.terminal import get_terminal_size
-from pandas.core.config import get_option, set_option, reset_option
+from pandas.core.config import get_option, set_option
 import pandas.core.common as com
 import pandas.lib as lib
-from pandas.tslib import iNaT
+from pandas.tslib import iNaT, Timestamp, Timedelta
 
 import numpy as np
 
 import itertools
 import csv
-from datetime import time
 
 from pandas.tseries.period import PeriodIndex, DatetimeIndex
 
@@ -93,22 +90,17 @@ class CategoricalFormatter(object):
                 footer += ', '
             footer += "Length: %d" % len(self.categorical)
 
-        levheader = 'Levels (%d): ' % len(self.categorical.levels)
+        level_info = self.categorical._repr_categories_info()
 
-        # TODO: should max_line_width respect a setting?
-        levstring = np.array_repr(self.categorical.levels, max_line_width=60)
-        indent = ' ' * (levstring.find('[') + len(levheader) + 1)
-        lines = levstring.split('\n')
-        levstring = '\n'.join([lines[0]] +
-                              [indent + x.lstrip() for x in lines[1:]])
+        # Levels are added in a newline
         if footer:
-            footer += ', '
-        footer += levheader + levstring
+            footer += '\n'
+        footer += level_info
 
         return compat.text_type(footer)
 
     def _get_formatted_values(self):
-        return format_array(np.asarray(self.categorical), None,
+        return format_array(self.categorical.get_values(), None,
                             float_format=None,
                             na_rep=self.na_rep)
 
@@ -122,9 +114,11 @@ class CategoricalFormatter(object):
                 return u('')
 
         fmt_values = self._get_formatted_values()
-        pad_space = 10
 
         result = ['%s' % i for i in fmt_values]
+        result = [i.strip() for i in result]
+        result = u(', ').join(result)
+        result = [u('[')+result+u(']')]
         if self.footer:
             footer = self._get_footer()
             if footer:
@@ -157,7 +151,9 @@ class SeriesFormatter(object):
                 footer += 'Freq: %s' % self.series.index.freqstr
 
             if footer and self.series.name is not None:
-                footer += ', '
+                # categories have already a comma + linebreak
+                if not com.is_categorical_dtype(self.series.dtype):
+                    footer += ', '
 
             series_name = com.pprint_thing(self.series.name,
                                            escape_chars=('\t', '\r', '\n'))
@@ -169,6 +165,7 @@ class SeriesFormatter(object):
                 footer += ', '
             footer += 'Length: %d' % len(self.series)
 
+        # TODO: in tidy_repr, with freq index, no dtype is shown -> also include a guard here?
         if self.dtype:
             name = getattr(self.series.dtype, 'name', None)
             if name:
@@ -176,6 +173,14 @@ class SeriesFormatter(object):
                     footer += ', '
                 footer += 'dtype: %s' % com.pprint_thing(name)
 
+        # level infos are added to the end and in a new line, like it is done for Categoricals
+        # Only added when we request a name
+        if self.name and com.is_categorical_dtype(self.series.dtype):
+            level_info = self.series.values._repr_categories_info()
+            if footer:
+                footer += "\n"
+            footer += level_info
+
         return compat.text_type(footer)
 
     def _get_formatted_index(self):
@@ -191,7 +196,7 @@ class SeriesFormatter(object):
         return fmt_index, have_header
 
     def _get_formatted_values(self):
-        return format_array(self.series.values, None,
+        return format_array(self.series.get_values(), None,
                             float_format=self.float_format,
                             na_rep=self.na_rep)
 
@@ -243,7 +248,8 @@ class TableFormatter(object):
 
     @property
     def should_show_dimensions(self):
-        return self.show_dimensions is True or (self.show_dimensions == 'truncate' and self.is_truncated)
+        return self.show_dimensions is True or (self.show_dimensions == 'truncate' and
+                                                self.is_truncated)
 
     def _get_formatter(self, i):
         if isinstance(self.formatters, (list, tuple)):
@@ -314,30 +320,69 @@ class DataFrameFormatter(TableFormatter):
         self._chk_truncate()
 
     def _chk_truncate(self):
+        '''
+        Checks whether the frame should be truncated. If so, slices
+        the frame up.
+        '''
         from pandas.tools.merge import concat
 
-        truncate_h = self.max_cols and (len(self.columns) > self.max_cols)
-        truncate_v = self.max_rows and (len(self.frame) > self.max_rows)
+        # Column of which first element is used to determine width of a dot col
+        self.tr_size_col = -1
 
         # Cut the data to the information actually printed
         max_cols = self.max_cols
         max_rows = self.max_rows
+
+        if max_cols == 0 or max_rows == 0:  # assume we are in the terminal (why else = 0)
+            (w, h) = get_terminal_size()
+            self.w = w
+            self.h = h
+            if self.max_rows == 0:
+                dot_row = 1
+                prompt_row = 1
+                if self.show_dimensions:
+                    show_dimension_rows = 3
+                n_add_rows = self.header + dot_row + show_dimension_rows + prompt_row
+                max_rows_adj = self.h - n_add_rows  # rows available to fill with actual data
+                self.max_rows_adj = max_rows_adj
+
+            # Format only rows and columns that could potentially fit the screen
+            if max_cols == 0 and len(self.frame.columns) > w:
+                max_cols = w
+            if max_rows == 0 and len(self.frame) > h:
+                max_rows = h
+
+        if not hasattr(self, 'max_rows_adj'):
+            self.max_rows_adj = max_rows
+        if not hasattr(self, 'max_cols_adj'):
+            self.max_cols_adj = max_cols
+
+        max_cols_adj = self.max_cols_adj
+        max_rows_adj = self.max_rows_adj
+
+        truncate_h = max_cols_adj and (len(self.columns) > max_cols_adj)
+        truncate_v = max_rows_adj and (len(self.frame) > max_rows_adj)
+
         frame = self.frame
         if truncate_h:
-            if max_cols > 1:
-                col_num = (max_cols // 2)
-                frame = concat( (frame.iloc[:,:col_num],frame.iloc[:,-col_num:]),axis=1 )
-            else:
+            if max_cols_adj == 0:
+                col_num = len(frame.columns)
+            elif max_cols_adj == 1:
+                frame = frame.iloc[:, :max_cols]
                 col_num = max_cols
-                frame = frame.iloc[:,:max_cols]
+            else:
+                col_num = (max_cols_adj // 2)
+                frame = concat((frame.iloc[:, :col_num], frame.iloc[:, -col_num:]), axis=1)
             self.tr_col_num = col_num
         if truncate_v:
-            if max_rows > 1:
-                row_num = max_rows // 2
-                frame = concat( (frame.iloc[:row_num,:],frame.iloc[-row_num:,:]) )
-            else:
+            if max_rows_adj == 0:
+                row_num = len(frame)
+            if max_rows_adj == 1:
                 row_num = max_rows
-                frame = frame.iloc[:max_rows,:]
+                frame = frame.iloc[:max_rows, :]
+            else:
+                row_num = max_rows_adj // 2
+                frame = concat((frame.iloc[:row_num, :], frame.iloc[-row_num:, :]))
             self.tr_row_num = row_num
 
         self.tr_frame = frame
@@ -353,13 +398,12 @@ class DataFrameFormatter(TableFormatter):
         frame = self.tr_frame
 
         # may include levels names also
-        str_index = self._get_formatted_index(frame)
 
+        str_index = self._get_formatted_index(frame)
         str_columns = self._get_formatted_column_labels(frame)
 
         if self.header:
             stringified = []
-            col_headers = frame.columns
             for i, c in enumerate(frame):
                 cheader = str_columns[i]
                 max_colwidth = max(self.col_space or 0,
@@ -370,7 +414,6 @@ class DataFrameFormatter(TableFormatter):
                 fmt_values = _make_fixed_width(fmt_values, self.justify,
                                                minimum=max_colwidth)
 
-
                 max_len = max(np.max([_strlen(x) for x in fmt_values]),
                               max_colwidth)
                 if self.justify == 'left':
@@ -382,9 +425,9 @@ class DataFrameFormatter(TableFormatter):
         else:
             stringified = []
             for i, c in enumerate(frame):
-                formatter = self._get_formatter(i)
                 fmt_values = self._format_col(i)
-                fmt_values = _make_fixed_width(fmt_values, self.justify)
+                fmt_values = _make_fixed_width(fmt_values, self.justify,
+                                               minimum=(self.col_space or 0))
 
                 stringified.append(fmt_values)
 
@@ -398,12 +441,12 @@ class DataFrameFormatter(TableFormatter):
 
         if truncate_h:
             col_num = self.tr_col_num
-            col_width = len(strcols[col_num][0])  # infer from column header
-            strcols.insert(col_num + 1, ['...'.center(col_width)] * (len(str_index)))
+            col_width = len(strcols[self.tr_size_col][0])  # infer from column header
+            strcols.insert(self.tr_col_num + 1, ['...'.center(col_width)] * (len(str_index)))
         if truncate_v:
             n_header_rows = len(str_index) - len(frame)
             row_num = self.tr_row_num
-            for ix,col in enumerate(strcols):
+            for ix, col in enumerate(strcols):
                 cwidth = len(strcols[ix][row_num])  # infer from above row
                 is_dot_col = False
                 if truncate_h:
@@ -416,19 +459,19 @@ class DataFrameFormatter(TableFormatter):
                 if ix == 0:
                     dot_str = my_str.ljust(cwidth)
                 elif is_dot_col:
+                    cwidth = len(strcols[self.tr_size_col][0])
                     dot_str = my_str.center(cwidth)
                 else:
                     dot_str = my_str.rjust(cwidth)
 
                 strcols[ix].insert(row_num + n_header_rows, dot_str)
-
         return strcols
 
     def to_string(self):
         """
         Render a DataFrame to a console-friendly tabular output.
         """
-
+        from pandas import Series
         frame = self.frame
 
         if len(frame.columns) == 0 or len(frame.index) == 0:
@@ -439,10 +482,40 @@ class DataFrameFormatter(TableFormatter):
             text = info_line
         else:
             strcols = self._to_str_columns()
-            if self.line_width is None:
+            if self.line_width is None:  # no need to wrap around just print the whole frame
                 text = adjoin(1, *strcols)
-            else:
+            elif not isinstance(self.max_cols, int) or self.max_cols > 0:  # need to wrap around
                 text = self._join_multiline(*strcols)
+            else:  # max_cols == 0. Try to fit frame to terminal
+                text = adjoin(1, *strcols).split('\n')
+                row_lens = Series(text).apply(len)
+                max_len_col_ix = np.argmax(row_lens)
+                max_len = row_lens[max_len_col_ix]
+                headers = [ele[0] for ele in strcols]
+                # Size of last col determines dot col size. See `self._to_str_columns
+                size_tr_col = len(headers[self.tr_size_col])
+                max_len += size_tr_col  # Need to make space for largest row plus truncate dot col
+                dif = max_len - self.w
+                adj_dif = dif
+                col_lens = Series([Series(ele).apply(len).max() for ele in strcols])
+                n_cols = len(col_lens)
+                counter = 0
+                while adj_dif > 0 and n_cols > 1:
+                    counter += 1
+                    mid = int(round(n_cols / 2.))
+                    mid_ix = col_lens.index[mid]
+                    col_len = col_lens[mid_ix]
+                    adj_dif -= (col_len + 1)  # adjoin adds one
+                    col_lens = col_lens.drop(mid_ix)
+                    n_cols = len(col_lens)
+                max_cols_adj = n_cols - self.index  # subtract index column
+                self.max_cols_adj = max_cols_adj
+
+                # Call again _chk_truncate to cut frame appropriately
+                # and then generate string representation
+                self._chk_truncate()
+                strcols = self._to_str_columns()
+                text = adjoin(1, *strcols)
 
         self.buf.writelines(text)
 
@@ -464,8 +537,8 @@ class DataFrameFormatter(TableFormatter):
         col_bins = _binify(col_widths, lwidth)
         nbins = len(col_bins)
 
-        if self.max_rows and len(self.frame) > self.max_rows:
-            nrows = self.max_rows + 1
+        if self.truncate_v:
+            nrows = self.max_rows_adj + 1
         else:
             nrows = len(self.frame)
 
@@ -489,7 +562,8 @@ class DataFrameFormatter(TableFormatter):
         Render a DataFrame to a LaTeX tabular/longtable environment output.
         """
         self.escape = self.kwds.get('escape', True)
-        #TODO: column_format is not settable in df.to_latex
+
+        # TODO: column_format is not settable in df.to_latex
         def get_col_type(dtype):
             if issubclass(dtype.type, np.number):
                 return 'r'
@@ -506,12 +580,22 @@ class DataFrameFormatter(TableFormatter):
         else:
             strcols = self._to_str_columns()
 
+        if self.index and isinstance(self.frame.index, MultiIndex):
+            clevels = self.frame.columns.nlevels
+            strcols.pop(0)
+            name = any(self.frame.columns.names)
+            for i, lev in enumerate(self.frame.index.levels):
+                lev2 = lev.format(name=name)
+                width = len(lev2[0])
+                lev3 = [' ' * width] * clevels + lev2
+                strcols.insert(i, lev3)
+
         if column_format is None:
             dtypes = self.frame.dtypes.values
+            column_format = ''.join(map(get_col_type, dtypes))
             if self.index:
-                column_format = 'l%s' % ''.join(map(get_col_type, dtypes))
-            else:
-                column_format = '%s' % ''.join(map(get_col_type, dtypes))
+                index_format = 'l' * self.frame.index.nlevels
+                column_format = index_format + column_format
         elif not isinstance(column_format,
                             compat.string_types):  # pragma: no cover
             raise AssertionError('column_format must be str or unicode, not %s'
@@ -525,7 +609,7 @@ class DataFrameFormatter(TableFormatter):
                 buf.write('\\begin{longtable}{%s}\n' % column_format)
                 buf.write('\\toprule\n')
 
-            nlevels = frame.index.nlevels
+            nlevels = frame.columns.nlevels
             for i, row in enumerate(zip(*strcols)):
                 if i == nlevels:
                     buf.write('\\midrule\n')  # End of header
@@ -539,7 +623,7 @@ class DataFrameFormatter(TableFormatter):
                         buf.write('\\bottomrule\n')
                         buf.write('\\endlastfoot\n')
                 if self.escape:
-                    crow = [(x.replace('\\', '\\textbackslash') # escape backslashes first
+                    crow = [(x.replace('\\', '\\textbackslash')  # escape backslashes first
                              .replace('_', '\\_')
                              .replace('%', '\\%')
                              .replace('$', '\\$')
@@ -594,7 +678,7 @@ class DataFrameFormatter(TableFormatter):
             raise TypeError('buf is not a file name and it has no write '
                             ' method')
 
-    def _get_formatted_column_labels(self,frame):
+    def _get_formatted_column_labels(self, frame):
         from pandas.core.index import _sparsify
 
         def is_numeric_dtype(dtype):
@@ -606,10 +690,17 @@ class DataFrameFormatter(TableFormatter):
             fmt_columns = columns.format(sparsify=False, adjoin=False)
             fmt_columns = lzip(*fmt_columns)
             dtypes = self.frame.dtypes.values
+
+            # if we have a Float level, they don't use leading space at all
+            restrict_formatting = any([l.is_floating for l in columns.levels])
             need_leadsp = dict(zip(fmt_columns, map(is_numeric_dtype, dtypes)))
-            str_columns = list(zip(*[
-                [' ' + y if y not in self.formatters and need_leadsp[x]
-                 else y for y in x] for x in fmt_columns]))
+
+            def space_format(x, y):
+                if y not in self.formatters and need_leadsp[x] and not restrict_formatting:
+                    return ' ' + y
+                return y
+
+            str_columns = list(zip(*[[space_format(x, y) for y in x] for x in fmt_columns]))
             if self.sparsify:
                 str_columns = _sparsify(str_columns)
 
@@ -628,6 +719,7 @@ class DataFrameFormatter(TableFormatter):
             for x in str_columns:
                 x.append('')
 
+        # self.str_columns = str_columns
         return str_columns
 
     @property
@@ -638,8 +730,8 @@ class DataFrameFormatter(TableFormatter):
     def has_column_names(self):
         return _has_names(self.frame.columns)
 
-    def _get_formatted_index(self,frame):
-        # Note: this is only used by to_string(), not by to_html().
+    def _get_formatted_index(self, frame):
+        # Note: this is only used by to_string() and to_latex(), not by to_html().
         index = frame.index
         columns = frame.columns
 
@@ -696,7 +788,8 @@ class HTMLFormatter(TableFormatter):
         self.max_rows = max_rows or len(self.fmt.frame)
         self.max_cols = max_cols or len(self.fmt.columns)
         self.show_dimensions = self.fmt.show_dimensions
-        self.is_truncated = self.max_rows < len(self.fmt.frame) or self.max_cols < len(self.fmt.columns)
+        self.is_truncated = (self.max_rows < len(self.fmt.frame) or
+                             self.max_cols < len(self.fmt.columns))
 
     def write(self, s, indent=0):
         rs = com.pprint_thing(s)
@@ -829,8 +922,8 @@ class HTMLFormatter(TableFormatter):
                     ins_col = self.fmt.tr_col_num
                     if self.fmt.sparsify:
                         recs_new = {}
-                        # Increment tags after ... col. 
-                        for tag,span in list(records.items()):
+                        # Increment tags after ... col.
+                        for tag, span in list(records.items()):
                             if tag >= ins_col:
                                 recs_new[tag + 1] = span
                             elif tag + span > ins_col:
@@ -839,12 +932,12 @@ class HTMLFormatter(TableFormatter):
                                     values = values[:ins_col] + (u('...'),) + \
                                         values[ins_col:]
                                 else:  # sparse col headers do not receive a ...
-                                    values = values[:ins_col] + \
-                                        (values[ins_col - 1],) + values[ins_col:]
+                                    values = (values[:ins_col] + (values[ins_col - 1],) +
+                                              values[ins_col:])
                             else:
                                 recs_new[tag] = span
                             # if ins_col lies between tags, all col headers get ...
-                            if tag + span == ins_col:  
+                            if tag + span == ins_col:
                                 recs_new[ins_col] = 1
                                 values = values[:ins_col] + (u('...'),) + \
                                     values[ins_col:]
@@ -854,7 +947,7 @@ class HTMLFormatter(TableFormatter):
                             records[ins_col] = 1
                     else:
                         recs_new = {}
-                        for tag,span in list(records.items()):
+                        for tag, span in list(records.items()):
                             if tag >= ins_col:
                                 recs_new[tag + 1] = span
                             else:
@@ -895,7 +988,7 @@ class HTMLFormatter(TableFormatter):
             ] + [''] * min(len(self.columns), self.max_cols)
             if truncate_h:
                 ins_col = row_levels + self.fmt.tr_col_num
-                row.insert(ins_col, '')                
+                row.insert(ins_col, '')
             self.write_tr(row, indent, self.indent_delta, header=True)
 
         indent -= self.indent_delta
@@ -940,10 +1033,11 @@ class HTMLFormatter(TableFormatter):
         else:
             index_values = self.fmt.tr_frame.index.format()
 
+        row = []
         for i in range(nrows):
 
             if truncate_v and i == (self.fmt.tr_row_num):
-                str_sep_row = [ '...' for ele in row ]
+                str_sep_row = ['...' for ele in row]
                 self.write_tr(str_sep_row, indent, self.indent_delta, tags=None,
                               nindex_levels=1)
 
@@ -967,44 +1061,42 @@ class HTMLFormatter(TableFormatter):
         nrows = len(frame)
         row_levels = self.frame.index.nlevels
 
-        idx_values = frame.index.format(sparsify=False, adjoin=False,
-                                                names=False)
+        idx_values = frame.index.format(sparsify=False, adjoin=False, names=False)
         idx_values = lzip(*idx_values)
 
         if self.fmt.sparsify:
             # GH3547
             sentinel = com.sentinel_factory()
-            levels = frame.index.format(sparsify=sentinel,
-                                                adjoin=False, names=False)
+            levels = frame.index.format(sparsify=sentinel, adjoin=False, names=False)
 
             level_lengths = _get_level_lengths(levels, sentinel)
             inner_lvl = len(level_lengths) - 1
             if truncate_v:
                 # Insert ... row and adjust idx_values and
-                # level_lengths to take this into account. 
+                # level_lengths to take this into account.
                 ins_row = self.fmt.tr_row_num
-                for lnum,records in enumerate(level_lengths):
+                for lnum, records in enumerate(level_lengths):
                     rec_new = {}
-                    for tag,span in list(records.items()):
+                    for tag, span in list(records.items()):
                         if tag >= ins_row:
                             rec_new[tag + 1] = span
                         elif tag + span > ins_row:
                             rec_new[tag] = span + 1
                             dot_row = list(idx_values[ins_row - 1])
                             dot_row[-1] = u('...')
-                            idx_values.insert(ins_row,tuple(dot_row))
+                            idx_values.insert(ins_row, tuple(dot_row))
                         else:
                             rec_new[tag] = span
                         # If ins_row lies between tags, all cols idx cols receive ...
                         if tag + span == ins_row:
                             rec_new[ins_row] = 1
                             if lnum == 0:
-                                idx_values.insert(ins_row,tuple([u('...')]*len(level_lengths)))                            
+                                idx_values.insert(ins_row, tuple([u('...')]*len(level_lengths)))
                     level_lengths[lnum] = rec_new
 
                 level_lengths[inner_lvl][ins_row] = 1
                 for ix_col in range(len(fmt_values)):
-                    fmt_values[ix_col].insert(ins_row,'...')
+                    fmt_values[ix_col].insert(ins_row, '...')
                 nrows += 1
 
             for i in range(nrows):
@@ -1042,6 +1134,7 @@ class HTMLFormatter(TableFormatter):
                 self.write_tr(row, indent, self.indent_delta, tags=None,
                               nindex_levels=frame.index.nlevels)
 
+
 def _get_level_lengths(levels, sentinel=''):
     from itertools import groupby
 
@@ -1179,7 +1272,7 @@ class CSVFormatter(object):
         if cols is None:
             cols = self.columns
 
-        has_aliases = isinstance(header, (tuple, list, np.ndarray))
+        has_aliases = isinstance(header, (tuple, list, np.ndarray, Index))
         if has_aliases or header:
             if index:
                 # should write something for index label
@@ -1198,7 +1291,7 @@ class CSVFormatter(object):
                             else:
                                 index_label = [index_label]
                     elif not isinstance(index_label,
-                                        (list, tuple, np.ndarray)):
+                                        (list, tuple, np.ndarray, Index)):
                         # given a string for a DF with Index
                         index_label = [index_label]
 
@@ -1222,10 +1315,10 @@ class CSVFormatter(object):
                 writer.writerow(encoded_cols)
 
         if date_format is None:
-            date_formatter = lambda x: lib.Timestamp(x)._repr_base
+            date_formatter = lambda x: Timestamp(x)._repr_base
         else:
             def strftime_with_nulls(x):
-                x = lib.Timestamp(x)
+                x = Timestamp(x)
                 if notnull(x):
                     return x.strftime(date_format)
 
@@ -1265,7 +1358,7 @@ class CSVFormatter(object):
 
                 if float_format is not None and com.is_float(val):
                     val = float_format % val
-                elif isinstance(val, (np.datetime64, lib.Timestamp)):
+                elif isinstance(val, (np.datetime64, Timestamp)):
                     val = date_formatter(val)
 
                 row_fields.append(val)
@@ -1295,7 +1388,7 @@ class CSVFormatter(object):
                 self.writer = csv.writer(f, **writer_kwargs)
 
             if self.engine == 'python':
-            # to be removed in 0.13
+                # to be removed in 0.13
                 self._helper_csv(self.writer, na_rep=self.na_rep,
                                  float_format=self.float_format,
                                  cols=self.cols, header=self.header,
@@ -1320,7 +1413,7 @@ class CSVFormatter(object):
         header = self.header
         encoded_labels = []
 
-        has_aliases = isinstance(header, (tuple, list, np.ndarray))
+        has_aliases = isinstance(header, (tuple, list, np.ndarray, Index))
         if not (has_aliases or self.header):
             return
         if has_aliases:
@@ -1348,7 +1441,7 @@ class CSVFormatter(object):
                             index_label = ['']
                         else:
                             index_label = [index_label]
-                elif not isinstance(index_label, (list, tuple, np.ndarray)):
+                elif not isinstance(index_label, (list, tuple, np.ndarray, Index)):
                     # given a string for a DF with Index
                     index_label = [index_label]
 
@@ -1505,15 +1598,15 @@ class ExcelFormatter(object):
             val = self.na_rep
         elif com.is_float(val):
             if np.isposinf(val):
-                val = '-%s' % self.inf_rep
-            elif np.isneginf(val):
                 val = self.inf_rep
+            elif np.isneginf(val):
+                val = '-%s' % self.inf_rep
             elif self.float_format is not None:
                 val = float(self.float_format % val)
         return val
 
     def _format_header_mi(self):
-        has_aliases = isinstance(self.header, (tuple, list, np.ndarray))
+        has_aliases = isinstance(self.header, (tuple, list, np.ndarray, Index))
         if not(has_aliases or self.header):
             return
 
@@ -1559,7 +1652,7 @@ class ExcelFormatter(object):
         self.rowcounter = lnum
 
     def _format_header_regular(self):
-        has_aliases = isinstance(self.header, (tuple, list, np.ndarray))
+        has_aliases = isinstance(self.header, (tuple, list, np.ndarray, Index))
         if has_aliases or self.header:
             coloffset = 0
 
@@ -1604,7 +1697,7 @@ class ExcelFormatter(object):
             return self._format_regular_rows()
 
     def _format_regular_rows(self):
-        has_aliases = isinstance(self.header, (tuple, list, np.ndarray))
+        has_aliases = isinstance(self.header, (tuple, list, np.ndarray, Index))
         if has_aliases or self.header:
             self.rowcounter += 1
 
@@ -1614,7 +1707,7 @@ class ExcelFormatter(object):
             # chek aliases
             # if list only take first as this is not a MultiIndex
             if self.index_label and isinstance(self.index_label,
-                                               (list, tuple, np.ndarray)):
+                                               (list, tuple, np.ndarray, Index)):
                 index_label = self.index_label[0]
             # if string good to go
             elif self.index_label and isinstance(self.index_label, str):
@@ -1654,7 +1747,7 @@ class ExcelFormatter(object):
                 yield ExcelCell(self.rowcounter + i, colidx + coloffset, val)
 
     def _format_hierarchical_rows(self):
-        has_aliases = isinstance(self.header, (tuple, list, np.ndarray))
+        has_aliases = isinstance(self.header, (tuple, list, np.ndarray, Index))
         if has_aliases or self.header:
             self.rowcounter += 1
 
@@ -1664,7 +1757,7 @@ class ExcelFormatter(object):
             index_labels = self.df.index.names
             # check for aliases
             if self.index_label and isinstance(self.index_label,
-                                               (list, tuple, np.ndarray)):
+                                               (list, tuple, np.ndarray, Index)):
                 index_labels = self.index_label
 
             # if index labels are not empty go ahead and dump
@@ -1731,7 +1824,7 @@ class ExcelFormatter(object):
             cell.val = self._format_value(cell.val)
             yield cell
 
-#----------------------------------------------------------------------
+# ----------------------------------------------------------------------
 # Array formatters
 
 
@@ -1900,10 +1993,10 @@ class Datetime64Formatter(GenericArrayFormatter):
         self.date_format = date_format
 
     def _format_strings(self):
-        formatter = self.formatter or _get_format_datetime64_from_values(
-                                                self.values,
-                                                nat_rep=self.nat_rep,
-                                                date_format=self.date_format)
+        formatter = (self.formatter or
+                     _get_format_datetime64_from_values(self.values,
+                                                        nat_rep=self.nat_rep,
+                                                        date_format=self.date_format))
 
         fmt_values = [formatter(x) for x in self.values]
 
@@ -1914,8 +2007,8 @@ def _format_datetime64(x, tz=None, nat_rep='NaT'):
     if x is None or lib.checknull(x):
         return nat_rep
 
-    if tz is not None or not isinstance(x, lib.Timestamp):
-        x = lib.Timestamp(x, tz=tz)
+    if tz is not None or not isinstance(x, Timestamp):
+        x = Timestamp(x, tz=tz)
 
     return str(x)
 
@@ -1924,8 +2017,8 @@ def _format_datetime64_dateonly(x, nat_rep='NaT', date_format=None):
     if x is None or lib.checknull(x):
         return nat_rep
 
-    if not isinstance(x, lib.Timestamp):
-        x = lib.Timestamp(x)
+    if not isinstance(x, Timestamp):
+        x = Timestamp(x)
 
     if date_format:
         return x.strftime(date_format)
@@ -1934,21 +2027,26 @@ def _format_datetime64_dateonly(x, nat_rep='NaT', date_format=None):
 
 
 def _is_dates_only(values):
-    for d in values:
-        if isinstance(d, np.datetime64):
-            d = lib.Timestamp(d)
-
-        if d is not None and not lib.checknull(d) and d._has_time_component():
-            return False
-    return True
+    # return a boolean if we are only dates (and don't have a timezone)
+    from pandas import DatetimeIndex
+    values = DatetimeIndex(values)
+    if values.tz is not None:
+        return False
 
+    values_int = values.asi8
+    consider_values = values_int != iNaT
+    one_day_nanos = (86400 * 1e9)
+    even_days = np.logical_and(consider_values, values_int % one_day_nanos != 0).sum() == 0
+    if even_days:
+        return True
+    return False
 
 def _get_format_datetime64(is_dates_only, nat_rep='NaT', date_format=None):
 
     if is_dates_only:
         return lambda x, tz=None: _format_datetime64_dateonly(x,
-                                nat_rep=nat_rep,
-                                date_format=date_format)
+                                                              nat_rep=nat_rep,
+                                                              date_format=date_format)
     else:
         return lambda x, tz=None: _format_datetime64(x, tz=tz, nat_rep=nat_rep)
 
@@ -1964,35 +2062,53 @@ def _get_format_datetime64_from_values(values,
 
 class Timedelta64Formatter(GenericArrayFormatter):
 
+    def __init__(self, values, nat_rep='NaT', box=False, **kwargs):
+        super(Timedelta64Formatter, self).__init__(values, **kwargs)
+        self.nat_rep = nat_rep
+        self.box = box
+
     def _format_strings(self):
-        formatter = self.formatter or _get_format_timedelta64(self.values)
-
+        formatter = self.formatter or _get_format_timedelta64(self.values, nat_rep=self.nat_rep,
+                                                              box=self.box)
         fmt_values = [formatter(x) for x in self.values]
-
         return fmt_values
 
 
-def _get_format_timedelta64(values):
+def _get_format_timedelta64(values, nat_rep='NaT', box=False):
+    """
+    Return a formatter function for a range of timedeltas.
+    These will all have the same format argument
+
+    If box, then show the return in quotes
+    """
+
     values_int = values.astype(np.int64)
 
     consider_values = values_int != iNaT
 
-    one_day_in_nanos = (86400 * 1e9)
-    even_days = np.logical_and(consider_values, values_int % one_day_in_nanos != 0).sum() == 0
-    all_sub_day = np.logical_and(consider_values, np.abs(values_int) >= one_day_in_nanos).sum() == 0
+    one_day_nanos = (86400 * 1e9)
+    even_days = np.logical_and(consider_values, values_int % one_day_nanos != 0).sum() == 0
+    all_sub_day = np.logical_and(consider_values, np.abs(values_int) >= one_day_nanos).sum() == 0
 
-    format_short = even_days or all_sub_day
-    format = "short" if format_short else "long"
+    if even_days:
+        format = 'even_day'
+    elif all_sub_day:
+        format = 'sub_day'
+    else:
+        format = 'long'
 
-    def impl(x):
+    def _formatter(x):
         if x is None or lib.checknull(x):
-            return 'NaT'
-        elif format_short and com.is_integer(x) and x.view('int64') == 0:
-            return "0 days" if even_days else "00:00:00"
-        else:
-            return lib.repr_timedelta64(x, format=format)
+            return nat_rep
 
-    return impl
+        if not isinstance(x, Timedelta):
+            x = Timedelta(x)
+        result = x._repr_base(format=format)
+        if box:
+            result = "'{0}'".format(result)
+        return result
+
+    return _formatter
 
 
 def _make_fixed_width(strings, justify='right', minimum=None):
@@ -2075,7 +2191,7 @@ def _has_names(index):
         return index.name is not None
 
 
-#------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 # Global formatting options
 
 _initial_defencoding = None
