@@ -15,8 +15,8 @@ import numpy as np
 
 from pandas.io.common import _is_url, urlopen, parse_url
 from pandas.io.parsers import TextParser
-from pandas.compat import (lrange, lmap, u, string_types, iteritems, text_type,
-                           raise_with_traceback)
+from pandas.compat import (lrange, lmap, u, string_types, iteritems,
+                           raise_with_traceback, binary_type)
 from pandas.core import common as com
 from pandas import Series
 
@@ -51,6 +51,9 @@ else:
 _RE_WHITESPACE = re.compile(r'[\r\n]+|\s{2,}')
 
 
+char_types = string_types + (binary_type,)
+
+
 def _remove_whitespace(s, regex=_RE_WHITESPACE):
     """Replace extra whitespace inside of a string with a single space.
 
@@ -98,30 +101,33 @@ def _get_skiprows(skiprows):
                     type(skiprows).__name__)
 
 
-def _read(io):
+def _read(obj):
     """Try to read from a url, file or string.
 
     Parameters
     ----------
-    io : str, unicode, or file-like
+    obj : str, unicode, or file-like
 
     Returns
     -------
     raw_text : str
     """
-    if _is_url(io):
-        with urlopen(io) as url:
-            raw_text = url.read()
-    elif hasattr(io, 'read'):
-        raw_text = io.read()
-    elif os.path.isfile(io):
-        with open(io) as f:
-            raw_text = f.read()
-    elif isinstance(io, string_types):
-        raw_text = io
+    if _is_url(obj):
+        with urlopen(obj) as url:
+            text = url.read()
+    elif hasattr(obj, 'read'):
+        text = obj.read()
+    elif isinstance(obj, char_types):
+        text = obj
+        try:
+            if os.path.isfile(text):
+                with open(text, 'rb') as f:
+                    return f.read()
+        except (TypeError, ValueError):
+            pass
     else:
-        raise TypeError("Cannot read object of type %r" % type(io).__name__)
-    return raw_text
+        raise TypeError("Cannot read object of type %r" % type(obj).__name__)
+    return text
 
 
 class _HtmlFrameParser(object):
@@ -165,10 +171,11 @@ class _HtmlFrameParser(object):
     See each method's respective documentation for details on their
     functionality.
     """
-    def __init__(self, io, match, attrs):
+    def __init__(self, io, match, attrs, encoding):
         self.io = io
         self.match = match
         self.attrs = attrs
+        self.encoding = encoding
 
     def parse_tables(self):
         tables = self._parse_tables(self._build_doc(), self.match, self.attrs)
@@ -422,7 +429,8 @@ class _BeautifulSoupHtml5LibFrameParser(_HtmlFrameParser):
 
     def _build_doc(self):
         from bs4 import BeautifulSoup
-        return BeautifulSoup(self._setup_build_doc(), features='html5lib')
+        return BeautifulSoup(self._setup_build_doc(), features='html5lib',
+                             from_encoding=self.encoding)
 
 
 def _build_xpath_expr(attrs):
@@ -519,7 +527,7 @@ class _LxmlFrameParser(_HtmlFrameParser):
         from lxml.html import parse, fromstring, HTMLParser
         from lxml.etree import XMLSyntaxError
 
-        parser = HTMLParser(recover=False)
+        parser = HTMLParser(recover=False, encoding=self.encoding)
 
         try:
             # try to parse the input in the simplest way
@@ -579,8 +587,9 @@ def _expand_elements(body):
     lens_max = lens.max()
     not_max = lens[lens != lens_max]
 
+    empty = ['']
     for ind, length in iteritems(not_max):
-        body[ind] += [np.nan] * (lens_max - length)
+        body[ind] += empty * (lens_max - length)
 
 
 def _data_to_frame(data, header, index_col, skiprows, infer_types,
@@ -601,11 +610,6 @@ def _data_to_frame(data, header, index_col, skiprows, infer_types,
                     parse_dates=parse_dates, tupleize_cols=tupleize_cols,
                     thousands=thousands)
     df = tp.read()
-
-    if infer_types:  # TODO: rm this code so infer_types has no effect in 0.14
-        df = df.convert_objects(convert_dates='coerce')
-    else:
-        df = df.applymap(text_type)
     return df
 
 
@@ -641,9 +645,9 @@ def _parser_dispatch(flavor):
 
     if flavor in ('bs4', 'html5lib'):
         if not _HAS_HTML5LIB:
-            raise ImportError("html5lib not found please install it")
+            raise ImportError("html5lib not found, please install it")
         if not _HAS_BS4:
-            raise ImportError("bs4 not found please install it")
+            raise ImportError("BeautifulSoup4 (bs4) not found, please install it")
         if bs4.__version__ == LooseVersion('4.2.0'):
             raise ValueError("You're using a version"
                              " of BeautifulSoup4 (4.2.0) that has been"
@@ -654,7 +658,7 @@ def _parser_dispatch(flavor):
                              " and later releases will work.")
     else:
         if not _HAS_LXML:
-            raise ImportError("lxml not found please install it")
+            raise ImportError("lxml not found, please install it")
     return _valid_parsers[flavor]
 
 
@@ -688,7 +692,7 @@ def _validate_flavor(flavor):
 
 
 def _parse(flavor, io, match, header, index_col, skiprows, infer_types,
-           parse_dates, tupleize_cols, thousands, attrs):
+           parse_dates, tupleize_cols, thousands, attrs, encoding):
     flavor = _validate_flavor(flavor)
     compiled_match = re.compile(match)  # you can pass a compiled regex here
 
@@ -696,7 +700,7 @@ def _parse(flavor, io, match, header, index_col, skiprows, infer_types,
     retained = None
     for flav in flavor:
         parser = _parser_dispatch(flav)
-        p = parser(io, compiled_match, attrs)
+        p = parser(io, compiled_match, attrs, encoding)
 
         try:
             tables = p.parse_tables()
@@ -707,14 +711,19 @@ def _parse(flavor, io, match, header, index_col, skiprows, infer_types,
     else:
         raise_with_traceback(retained)
 
-    return [_data_to_frame(table, header, index_col, skiprows, infer_types,
-                           parse_dates, tupleize_cols, thousands)
-            for table in tables]
+    ret = []
+    for table in tables:
+        try:
+            ret.append(_data_to_frame(table, header, index_col, skiprows,
+                        infer_types, parse_dates, tupleize_cols, thousands))
+        except StopIteration: # empty table
+            continue
+    return ret
 
 
 def read_html(io, match='.+', flavor=None, header=None, index_col=None,
               skiprows=None, infer_types=None, attrs=None, parse_dates=False,
-              tupleize_cols=False, thousands=','):
+              tupleize_cols=False, thousands=',', encoding=None):
     r"""Read HTML tables into a ``list`` of ``DataFrame`` objects.
 
     Parameters
@@ -751,24 +760,23 @@ def read_html(io, match='.+', flavor=None, header=None, index_col=None,
         that sequence.  Note that a single element sequence means 'skip the nth
         row' whereas an integer means 'skip n rows'.
 
-    infer_types : bool, optional
-        This option is deprecated in 0.13, an will have no effect in 0.14. It
-        defaults to ``True``.
+    infer_types : None, optional
+        This has no effect since 0.15.0. It is here for backwards compatibility.
 
     attrs : dict or None, optional
         This is a dictionary of attributes that you can pass to use to identify
         the table in the HTML. These are not checked for validity before being
         passed to lxml or Beautiful Soup. However, these attributes must be
         valid HTML table attributes to work correctly. For example, ::
-        
+
             attrs = {'id': 'table'}
-        
+
         is a valid attribute dictionary because the 'id' HTML tag attribute is
         a valid HTML attribute for *any* HTML tag as per `this document
         <http://www.w3.org/TR/html-markup/global-attributes.html>`__. ::
-        
+
             attrs = {'asdf': 'table'}
-        
+
         is *not* a valid attribute dictionary because 'asdf' is not a valid
         HTML attribute even if it is a valid XML attribute.  Valid HTML 4.01
         table attributes can be found `here
@@ -791,6 +799,12 @@ def read_html(io, match='.+', flavor=None, header=None, index_col=None,
     thousands : str, optional
         Separator to use to parse thousands. Defaults to ``','``.
 
+    encoding : str or None, optional
+        The encoding used to decode the web page. Defaults to ``None``.``None``
+        preserves the previous encoding behavior, which depends on the
+        underlying parser library (e.g., the parser library will try to use
+        the encoding provided by the document).
+
     Returns
     -------
     dfs : list of DataFrames
@@ -826,9 +840,7 @@ def read_html(io, match='.+', flavor=None, header=None, index_col=None,
     pandas.io.parsers.read_csv
     """
     if infer_types is not None:
-        warnings.warn("infer_types will have no effect in 0.14", FutureWarning)
-    else:
-        infer_types = True  # TODO: remove effect of this in 0.14
+        warnings.warn("infer_types has no effect since 0.15", FutureWarning)
 
     # Type check here. We don't want to parse only to fail because of an
     # invalid value of an integer skiprows.
@@ -836,4 +848,4 @@ def read_html(io, match='.+', flavor=None, header=None, index_col=None,
         raise ValueError('cannot skip rows starting from the end of the '
                          'data (you passed a negative value)')
     return _parse(flavor, io, match, header, index_col, skiprows, infer_types,
-                  parse_dates, tupleize_cols, thousands, attrs)
+                  parse_dates, tupleize_cols, thousands, attrs, encoding)
