@@ -1,6 +1,5 @@
 from datetime import datetime
 from pandas.compat import range
-import pickle
 import nose
 import sys
 import numpy as np
@@ -168,9 +167,7 @@ class TestDateRange(tm.TestCase):
         self.assertEqual(shifted[0], rng[0] + datetools.bday)
 
     def test_pickle_unpickle(self):
-        pickled = pickle.dumps(self.rng)
-        unpickled = pickle.loads(pickled)
-
+        unpickled = self.round_trip_pickle(self.rng)
         self.assertIsNotNone(unpickled.offset)
 
     def test_union(self):
@@ -354,25 +351,51 @@ class TestDateRange(tm.TestCase):
     def test_range_tz_pytz(self):
         # GH 2906
         tm._skip_if_no_pytz()
-        from pytz import timezone as tz
+        from pytz import timezone
 
-        start = datetime(2011, 1, 1, tzinfo=tz('US/Eastern'))
-        end = datetime(2011, 1, 3, tzinfo=tz('US/Eastern'))
+        tz = timezone('US/Eastern')
+        start = tz.localize(datetime(2011, 1, 1))
+        end = tz.localize(datetime(2011, 1, 3))
 
         dr = date_range(start=start, periods=3)
-        self.assertEqual(dr.tz, tz('US/Eastern'))
+        self.assertEqual(dr.tz.zone, tz.zone)
         self.assertEqual(dr[0], start)
         self.assertEqual(dr[2], end)
 
         dr = date_range(end=end, periods=3)
-        self.assertEqual(dr.tz, tz('US/Eastern'))
+        self.assertEqual(dr.tz.zone, tz.zone)
         self.assertEqual(dr[0], start)
         self.assertEqual(dr[2], end)
 
         dr = date_range(start=start, end=end)
-        self.assertEqual(dr.tz, tz('US/Eastern'))
+        self.assertEqual(dr.tz.zone, tz.zone)
         self.assertEqual(dr[0], start)
         self.assertEqual(dr[2], end)
+    
+    def test_range_tz_dst_straddle_pytz(self):
+        
+        tm._skip_if_no_pytz()
+        from pytz import timezone
+        tz = timezone('US/Eastern')
+        dates = [(tz.localize(datetime(2014, 3, 6)), 
+                  tz.localize(datetime(2014, 3, 12))),
+                 (tz.localize(datetime(2013, 11, 1)), 
+                  tz.localize(datetime(2013, 11, 6)))]
+        for (start, end) in dates:
+            dr = date_range(start, end, freq='D')
+            self.assertEqual(dr[0], start)
+            self.assertEqual(dr[-1], end)
+            self.assertEqual(np.all(dr.hour==0), True)
+            
+            dr = date_range(start, end, freq='D', tz='US/Eastern')
+            self.assertEqual(dr[0], start)
+            self.assertEqual(dr[-1], end)
+            self.assertEqual(np.all(dr.hour==0), True)        
+            
+            dr = date_range(start.replace(tzinfo=None), end.replace(tzinfo=None), freq='D', tz='US/Eastern')
+            self.assertEqual(dr[0], start)
+            self.assertEqual(dr[-1], end)
+            self.assertEqual(np.all(dr.hour==0), True)        
 
     def test_range_tz_dateutil(self):
         # GH 2906
@@ -535,9 +558,7 @@ class TestCustomDateRange(tm.TestCase):
         self.assertEqual(shifted[0], rng[0] + datetools.cday)
 
     def test_pickle_unpickle(self):
-        pickled = pickle.dumps(self.rng)
-        unpickled = pickle.loads(pickled)
-
+        unpickled = self.round_trip_pickle(self.rng)
         self.assertIsNotNone(unpickled.offset)
 
     def test_union(self):
