@@ -17,12 +17,7 @@ from pandas.tseries.resample import DatetimeIndex
 from pandas.util.testing import assert_series_equal, ensure_clean
 import pandas.util.testing as tm
 
-
-def _skip_if_no_scipy():
-    try:
-        import scipy
-    except ImportError:
-        raise nose.SkipTest("scipy not installed")
+from pandas.tests.test_graphics import _skip_if_no_scipy_gaussian_kde
 
 
 @tm.mplskip
@@ -64,7 +59,7 @@ class TestTSPlot(tm.TestCase):
         _check_plot_works(df.plot)
 
         # axes freq
-        idx = idx[0:40] + idx[45:99]
+        idx = idx[0:40].union(idx[45:99])
         df2 = DataFrame(np.random.randn(len(idx), 3), index=idx)
         _check_plot_works(df2.plot)
 
@@ -81,7 +76,7 @@ class TestTSPlot(tm.TestCase):
         df = DataFrame({'A': ["x", "y", "z"], 'B': [1,2,3]}, idx)
 
         ax = df.plot() # it works
-        self.assert_(len(ax.get_lines()) == 1) #B was plotted
+        self.assertEqual(len(ax.get_lines()), 1) #B was plotted
         plt.close(plt.gcf())
 
         self.assertRaises(TypeError, df['A'].plot)
@@ -124,13 +119,31 @@ class TestTSPlot(tm.TestCase):
 
     def test_get_datevalue(self):
         from pandas.tseries.converter import get_datevalue
-        self.assert_(get_datevalue(None, 'D') is None)
+        self.assertIsNone(get_datevalue(None, 'D'))
         self.assertEqual(get_datevalue(1987, 'A'), 1987)
         self.assertEqual(get_datevalue(Period(1987, 'A'), 'M'),
                          Period('1987-12', 'M').ordinal)
         self.assertEqual(get_datevalue('1/1/1987', 'D'),
                          Period('1987-1-1', 'D').ordinal)
 
+    @slow
+    def test_ts_plot_format_coord(self):
+        def check_format_of_first_point(ax, expected_string):
+            first_line = ax.get_lines()[0]
+            first_x = first_line.get_xdata()[0].ordinal
+            first_y = first_line.get_ydata()[0]
+            try:
+                self.assertEqual(expected_string, ax.format_coord(first_x, first_y))
+            except (ValueError):
+                raise nose.SkipTest("skipping test because issue forming test comparison GH7664")
+
+        annual = Series(1, index=date_range('2014-01-01', periods=3, freq='A-DEC'))
+        check_format_of_first_point(annual.plot(), 't = 2014  y = 1.000000')
+
+        # note this is added to the annual plot already in existence, and changes its freq field
+        daily = Series(1, index=date_range('2014-01-01', periods=3, freq='D'))
+        check_format_of_first_point(daily.plot(), 't = 2014-01-01  y = 1.000000')
+
     @slow
     def test_line_plot_period_series(self):
         for s in self.period_ser:
@@ -226,7 +239,7 @@ class TestTSPlot(tm.TestCase):
         diffs = Series(ax.get_lines()[0].get_xydata()[:, 0]).diff()
 
         sec = 1. / 24 / 60 / 60
-        self.assert_((np.fabs(diffs[1:] - [sec, sec * 2, sec]) < 1e-8).all())
+        self.assertTrue((np.fabs(diffs[1:] - [sec, sec * 2, sec]) < 1e-8).all())
 
         plt.clf()
         fig.add_subplot(111)
@@ -234,7 +247,7 @@ class TestTSPlot(tm.TestCase):
         df2.index = df.index.asobject
         ax = df2.plot()
         diffs = Series(ax.get_lines()[0].get_xydata()[:, 0]).diff()
-        self.assert_((np.fabs(diffs[1:] - sec) < 1e-8).all())
+        self.assertTrue((np.fabs(diffs[1:] - sec) < 1e-8).all())
 
     def test_irregular_datetime64_repr_bug(self):
         import matplotlib.pyplot as plt
@@ -245,7 +258,7 @@ class TestTSPlot(tm.TestCase):
         plt.clf()
         ax = fig.add_subplot(211)
         ret = ser.plot()
-        self.assert_(ret is not None)
+        self.assertIsNotNone(ret)
 
         for rs, xp in zip(ax.get_lines()[0].get_xdata(), ser.index):
             self.assertEqual(rs, xp)
@@ -285,7 +298,7 @@ class TestTSPlot(tm.TestCase):
         bts = DataFrame({'a': tm.makeTimeSeries()})
         ax = bts.plot()
         idx = ax.get_lines()[0].get_xdata()
-        assert_array_equal(bts.index.to_period(), idx)
+        assert_array_equal(bts.index.to_period(), PeriodIndex(idx))
 
     @slow
     def test_axis_limits(self):
@@ -390,7 +403,7 @@ class TestTSPlot(tm.TestCase):
             ax = ser.plot()
             xaxis = ax.get_xaxis()
             rs = xaxis.get_majorticklocs()[0]
-            self.assert_(rs == xp)
+            self.assertEqual(rs, xp)
             vmin, vmax = ax.get_xlim()
             ax.set_xlim(vmin + 0.9, vmax)
             rs = xaxis.get_majorticklocs()[0]
@@ -453,7 +466,7 @@ class TestTSPlot(tm.TestCase):
         data = l.get_xydata()
         tm.assert_isinstance(data, np.ma.core.MaskedArray)
         mask = data.mask
-        self.assert_(mask[5:25, 1].all())
+        self.assertTrue(mask[5:25, 1].all())
         plt.close(ax.get_figure())
 
         # irregular
@@ -467,7 +480,7 @@ class TestTSPlot(tm.TestCase):
         data = l.get_xydata()
         tm.assert_isinstance(data, np.ma.core.MaskedArray)
         mask = data.mask
-        self.assert_(mask[2:5, 1].all())
+        self.assertTrue(mask[2:5, 1].all())
         plt.close(ax.get_figure())
 
         # non-ts
@@ -481,7 +494,7 @@ class TestTSPlot(tm.TestCase):
         data = l.get_xydata()
         tm.assert_isinstance(data, np.ma.core.MaskedArray)
         mask = data.mask
-        self.assert_(mask[2:5, 1].all())
+        self.assertTrue(mask[2:5, 1].all())
 
     @slow
     def test_gap_upsample(self):
@@ -499,7 +512,7 @@ class TestTSPlot(tm.TestCase):
         data = l.get_xydata()
         tm.assert_isinstance(data, np.ma.core.MaskedArray)
         mask = data.mask
-        self.assert_(mask[5:25, 1].all())
+        self.assertTrue(mask[5:25, 1].all())
 
     @slow
     def test_secondary_y(self):
@@ -523,7 +536,7 @@ class TestTSPlot(tm.TestCase):
 
         ax = ser2.plot()
         ax2 = ser.plot(secondary_y=True).right_ax
-        self.assert_(ax.get_yaxis().get_visible())
+        self.assertTrue(ax.get_yaxis().get_visible())
 
     @slow
     def test_secondary_y_ts(self):
@@ -547,11 +560,12 @@ class TestTSPlot(tm.TestCase):
 
         ax = ser2.plot()
         ax2 = ser.plot(secondary_y=True)
-        self.assert_(ax.get_yaxis().get_visible())
+        self.assertTrue(ax.get_yaxis().get_visible())
 
     @slow
     def test_secondary_kde(self):
-        _skip_if_no_scipy()
+        tm._skip_if_no_scipy()
+        _skip_if_no_scipy_gaussian_kde()
 
         import matplotlib.pyplot as plt
         ser = Series(np.random.randn(10))
@@ -591,10 +605,10 @@ class TestTSPlot(tm.TestCase):
         ax = s1.plot()
         ax2 = s2.plot(style='g')
         lines = ax2.get_lines()
-        idx1 = lines[0].get_xdata()
-        idx2 = lines[1].get_xdata()
-        self.assert_(idx1.equals(s1.index.to_period('B')))
-        self.assert_(idx2.equals(s2.index.to_period('B')))
+        idx1 = PeriodIndex(lines[0].get_xdata())
+        idx2 = PeriodIndex(lines[1].get_xdata())
+        self.assertTrue(idx1.equals(s1.index.to_period('B')))
+        self.assertTrue(idx2.equals(s2.index.to_period('B')))
         left, right = ax2.get_xlim()
         pidx = s1.index.to_period()
         self.assertEqual(left, pidx[0].ordinal)
@@ -680,7 +694,7 @@ class TestTSPlot(tm.TestCase):
         high.plot()
         ax = low.plot()
         for l in ax.get_lines():
-            self.assert_(PeriodIndex(data=l.get_xdata()).freq.startswith('W'))
+            self.assertTrue(PeriodIndex(data=l.get_xdata()).freq.startswith('W'))
 
     @slow
     def test_from_weekly_resampling(self):
@@ -690,8 +704,102 @@ class TestTSPlot(tm.TestCase):
         low = Series(np.random.randn(len(idxl)), idxl)
         low.plot()
         ax = high.plot()
+
+        expected_h = idxh.to_period().asi8
+        expected_l = np.array([1514, 1519, 1523, 1527, 1531, 1536, 1540, 1544, 1549,
+                               1553, 1558, 1562])
         for l in ax.get_lines():
-            self.assert_(PeriodIndex(data=l.get_xdata()).freq.startswith('W'))
+            self.assertTrue(PeriodIndex(data=l.get_xdata()).freq.startswith('W'))
+
+            xdata = l.get_xdata(orig=False)
+            if len(xdata) == 12: # idxl lines
+                self.assert_numpy_array_equal(xdata, expected_l)
+            else:
+                self.assert_numpy_array_equal(xdata, expected_h)
+
+    @slow
+    def test_from_resampling_area_line_mixed(self):
+        idxh = date_range('1/1/1999', periods=52, freq='W')
+        idxl = date_range('1/1/1999', periods=12, freq='M')
+        high = DataFrame(np.random.rand(len(idxh), 3),
+                         index=idxh, columns=[0, 1, 2])
+        low = DataFrame(np.random.rand(len(idxl), 3),
+                     index=idxl, columns=[0, 1, 2])
+
+        # low to high
+        for kind1, kind2 in [('line', 'area'), ('area', 'line')]:
+            ax = low.plot(kind=kind1, stacked=True)
+            ax = high.plot(kind=kind2, stacked=True, ax=ax)
+
+            # check low dataframe result
+            expected_x = np.array([1514, 1519, 1523, 1527, 1531, 1536, 1540, 1544, 1549,
+                                   1553, 1558, 1562])
+            expected_y = np.zeros(len(expected_x))
+            for i in range(3):
+                l = ax.lines[i]
+                self.assertTrue(PeriodIndex(data=l.get_xdata()).freq.startswith('W'))
+                self.assert_numpy_array_equal(l.get_xdata(orig=False), expected_x)
+                # check stacked values are correct
+                expected_y += low[i].values
+                self.assert_numpy_array_equal(l.get_ydata(orig=False), expected_y)
+
+            # check high dataframe result
+            expected_x = idxh.to_period().asi8
+            expected_y = np.zeros(len(expected_x))
+            for i in range(3):
+                l = ax.lines[3 + i]
+                self.assertTrue(PeriodIndex(data=l.get_xdata()).freq.startswith('W'))
+                self.assert_numpy_array_equal(l.get_xdata(orig=False), expected_x)
+                expected_y += high[i].values
+                self.assert_numpy_array_equal(l.get_ydata(orig=False), expected_y)
+
+        # high to low
+        for kind1, kind2 in [('line', 'area'), ('area', 'line')]:
+            ax = high.plot(kind=kind1, stacked=True)
+            ax = low.plot(kind=kind2, stacked=True, ax=ax)
+
+            # check high dataframe result
+            expected_x = idxh.to_period().asi8
+            expected_y = np.zeros(len(expected_x))
+            for i in range(3):
+                l = ax.lines[i]
+                self.assertTrue(PeriodIndex(data=l.get_xdata()).freq.startswith('W'))
+                self.assert_numpy_array_equal(l.get_xdata(orig=False), expected_x)
+                expected_y += high[i].values
+                self.assert_numpy_array_equal(l.get_ydata(orig=False), expected_y)
+
+            # check low dataframe result
+            expected_x = np.array([1514, 1519, 1523, 1527, 1531, 1536, 1540, 1544, 1549,
+                                   1553, 1558, 1562])
+            expected_y = np.zeros(len(expected_x))
+            for i in range(3):
+                l = ax.lines[3 + i]
+                self.assertTrue(PeriodIndex(data=l.get_xdata()).freq.startswith('W'))
+                self.assert_numpy_array_equal(l.get_xdata(orig=False), expected_x)
+                expected_y += low[i].values
+                self.assert_numpy_array_equal(l.get_ydata(orig=False), expected_y)
+
+    @slow
+    def test_mixed_freq_second_millisecond(self):
+        # GH 7772, GH 7760
+        idxh = date_range('2014-07-01 09:00', freq='S', periods=50)
+        idxl = date_range('2014-07-01 09:00', freq='100L', periods=500)
+        high = Series(np.random.randn(len(idxh)), idxh)
+        low = Series(np.random.randn(len(idxl)), idxl)
+        # high to low
+        high.plot()
+        ax = low.plot()
+        self.assertEqual(len(ax.get_lines()), 2)
+        for l in ax.get_lines():
+            self.assertEqual(PeriodIndex(data=l.get_xdata()).freq, 'L')
+        tm.close()
+
+        # low to high
+        low.plot()
+        ax = high.plot()
+        self.assertEqual(len(ax.get_lines()), 2)
+        for l in ax.get_lines():
+            self.assertEqual(PeriodIndex(data=l.get_xdata()).freq, 'L')
 
     @slow
     def test_irreg_dtypes(self):
@@ -773,9 +881,9 @@ class TestTSPlot(tm.TestCase):
         low.plot()
         ax = high.plot(secondary_y=True)
         for l in ax.get_lines():
-            self.assertEqual(l.get_xdata().freq, 'D')
+            self.assertEqual(PeriodIndex(l.get_xdata()).freq, 'D')
         for l in ax.right_ax.get_lines():
-            self.assertEqual(l.get_xdata().freq, 'D')
+            self.assertEqual(PeriodIndex(l.get_xdata()).freq, 'D')
 
     @slow
     def test_secondary_legend(self):
@@ -793,7 +901,7 @@ class TestTSPlot(tm.TestCase):
         self.assertEqual(leg.get_texts()[1].get_text(), 'B (right)')
         self.assertEqual(leg.get_texts()[2].get_text(), 'C')
         self.assertEqual(leg.get_texts()[3].get_text(), 'D')
-        self.assert_(ax.right_ax.get_legend() is None)
+        self.assertIsNone(ax.right_ax.get_legend())
         colors = set()
         for line in leg.get_lines():
             colors.add(line.get_color())
@@ -829,7 +937,7 @@ class TestTSPlot(tm.TestCase):
         ax = df.plot(secondary_y=['C', 'D'])
         leg = ax.get_legend()
         self.assertEqual(len(leg.get_lines()), 4)
-        self.assert_(ax.right_ax.get_legend() is None)
+        self.assertIsNone(ax.right_ax.get_legend())
         colors = set()
         for line in leg.get_lines():
             colors.add(line.get_color())
@@ -844,7 +952,7 @@ class TestTSPlot(tm.TestCase):
         ax = df.plot(secondary_y=['A', 'B'])
         leg = ax.get_legend()
         self.assertEqual(len(leg.get_lines()), 4)
-        self.assert_(ax.right_ax.get_legend() is None)
+        self.assertIsNone(ax.right_ax.get_legend())
         colors = set()
         for line in leg.get_lines():
             colors.add(line.get_color())
@@ -857,7 +965,7 @@ class TestTSPlot(tm.TestCase):
         ax = df.plot(secondary_y=['C', 'D'])
         leg = ax.get_legend()
         self.assertEqual(len(leg.get_lines()), 4)
-        self.assert_(ax.right_ax.get_legend() is None)
+        self.assertIsNone(ax.right_ax.get_legend())
         colors = set()
         for line in leg.get_lines():
             colors.add(line.get_color())
@@ -908,6 +1016,84 @@ class TestTSPlot(tm.TestCase):
         assert_array_equal(np.array([x.toordinal() for x in dates]),
                            line2.get_xydata()[:, 0])
 
+    @slow
+    def test_irregular_ts_shared_ax_xlim(self):
+        # GH 2960
+        ts = tm.makeTimeSeries()[:20]
+        ts_irregular = ts[[1, 4, 5, 6, 8, 9, 10, 12, 13, 14, 15, 17, 18]]
+
+        # plot the left section of the irregular series, then the right section
+        ax = ts_irregular[:5].plot()
+        ts_irregular[5:].plot(ax=ax)
+
+        # check that axis limits are correct
+        left, right = ax.get_xlim()
+        self.assertEqual(left, ts_irregular.index.min().toordinal())
+        self.assertEqual(right, ts_irregular.index.max().toordinal())
+
+    @slow
+    def test_secondary_y_non_ts_xlim(self):
+        # GH 3490 - non-timeseries with secondary y
+        index_1 = [1, 2, 3, 4]
+        index_2 = [5, 6, 7, 8]
+        s1 = Series(1, index=index_1)
+        s2 = Series(2, index=index_2)
+
+        ax = s1.plot()
+        left_before, right_before = ax.get_xlim()
+        s2.plot(secondary_y=True, ax=ax)
+        left_after, right_after = ax.get_xlim()
+
+        self.assertEqual(left_before, left_after)
+        self.assertTrue(right_before < right_after)
+
+    @slow
+    def test_secondary_y_regular_ts_xlim(self):
+        # GH 3490 - regular-timeseries with secondary y
+        index_1 = date_range(start='2000-01-01', periods=4, freq='D')
+        index_2 = date_range(start='2000-01-05', periods=4, freq='D')
+        s1 = Series(1, index=index_1)
+        s2 = Series(2, index=index_2)
+
+        ax = s1.plot()
+        left_before, right_before = ax.get_xlim()
+        s2.plot(secondary_y=True, ax=ax)
+        left_after, right_after = ax.get_xlim()
+
+        self.assertEqual(left_before, left_after)
+        self.assertTrue(right_before < right_after)
+
+    @slow
+    def test_secondary_y_mixed_freq_ts_xlim(self):
+        # GH 3490 - mixed frequency timeseries with secondary y
+        rng = date_range('2000-01-01', periods=10000, freq='min')
+        ts = Series(1, index=rng)
+
+        ax = ts.plot()
+        left_before, right_before = ax.get_xlim()
+        ts.resample('D').plot(secondary_y=True, ax=ax)
+        left_after, right_after = ax.get_xlim()
+
+        # a downsample should not have changed either limit
+        self.assertEqual(left_before, left_after)
+        self.assertEqual(right_before, right_after)
+
+    @slow
+    def test_secondary_y_irregular_ts_xlim(self):
+        # GH 3490 - irregular-timeseries with secondary y
+        ts = tm.makeTimeSeries()[:20]
+        ts_irregular = ts[[1, 4, 5, 6, 8, 9, 10, 12, 13, 14, 15, 17, 18]]
+
+        ax = ts_irregular[:5].plot()
+        # plot higher-x values on secondary axis
+        ts_irregular[5:].plot(secondary_y=True, ax=ax)
+        # ensure secondary limits aren't overwritten by plot on primary
+        ts_irregular[:5].plot(ax=ax)
+
+        left, right = ax.get_xlim()
+        self.assertEqual(left, ts_irregular.index.min().toordinal())
+        self.assertEqual(right, ts_irregular.index.max().toordinal())
+
 
 def _check_plot_works(f, freq=None, series=None, *args, **kwargs):
     import matplotlib.pyplot as plt
