@@ -21,6 +21,7 @@ from distutils.version import LooseVersion
 
 from numpy.random import randn, rand
 import numpy as np
+from numpy.testing import assert_array_equal
 
 import pandas as pd
 from pandas.core.common import _is_sequence, array_equivalent
@@ -39,9 +40,10 @@ from pandas.computation import expressions as expr
 
 from pandas import bdate_range
 from pandas.tseries.index import DatetimeIndex
+from pandas.tseries.tdi import TimedeltaIndex
 from pandas.tseries.period import PeriodIndex
 
-from pandas import _testing, _np_version_under1p7
+from pandas import _testing
 
 
 from pandas.io.common import urlopen
@@ -98,7 +100,14 @@ class TestCase(unittest.TestCase):
             return
         raise AssertionError('{0} is not equal to {1}.'.format(np_array, assert_equal))
 
-    def assert_numpy_array_equivalent(self, np_array, assert_equal):
+    def round_trip_pickle(self, obj, path=None):
+        if path is None:
+            path = u('__%s__.pickle' % rands(10))
+        with ensure_clean(path) as path:
+            pd.to_pickle(obj, path)
+            return pd.read_pickle(path)
+
+    def assert_numpy_array_equivalent(self, np_array, assert_equal, strict_nan=False):
         """Checks that 'np_array' is equivalent to 'assert_equal'
 
         Two numpy arrays are equivalent if the arrays have equal non-NaN elements, and
@@ -108,10 +117,20 @@ class TestCase(unittest.TestCase):
         similar to `assert_numpy_array_equal()`. If the expected array includes `np.nan` use this
         function.
         """
-        if array_equivalent(np_array, assert_equal):
+        if array_equivalent(np_array, assert_equal, strict_nan=strict_nan):
             return
         raise AssertionError('{0} is not equivalent to {1}.'.format(np_array, assert_equal))
 
+    def assert_categorical_equal(self, res, exp):
+        if not array_equivalent(res.categories, exp.categories):
+            raise AssertionError('categories not equivalent: {0} vs {1}.'.format(res.categories,
+                                                                                 exp.categories))
+        if not array_equivalent(res.codes, exp.codes):
+            raise AssertionError('codes not equivalent: {0} vs {1}.'.format(res.codes,
+                                                                            exp.codes))
+        self.assertEqual(res.ordered, exp.ordered, "ordered not the same")
+        self.assertEqual(res.name, exp.name, "name not the same")
+
     def assertIs(self, first, second, msg=''):
         """Checks that 'first' is 'second'"""
         a, b = first, second
@@ -156,6 +175,14 @@ class TestCase(unittest.TestCase):
             "%sExpected object to be of type %r, found %r instead" % (
                 msg, cls, type(obj)))
 
+    def assertRaises(self, _exception, _callable=None, *args, **kwargs):
+        """ compat with 2.6; assert that an exception is raised """
+        assertRaises(_exception, _callable, *args, **kwargs)
+
+    def assertRaisesRegexp(self, _exception, _regexp, _callable=None, *args, **kwargs):
+        """ Port of assertRaisesRegexp from unittest in Python 2.7 - used in with statement """
+        assertRaisesRegexp(_exception, _regexp, _callable, *args, **kwargs)
+
 # NOTE: don't pass an NDFrame or index to this function - may not handle it
 # well.
 assert_almost_equal = _testing.assert_almost_equal
@@ -210,11 +237,6 @@ def mplskip(cls):
     cls.setUpClass = setUpClass
     return cls
 
-def _skip_if_not_numpy17_friendly():
-    # not friendly for < 1.7
-    if _np_version_under1p7:
-        import nose
-        raise nose.SkipTest("numpy >= 1.7 is required")
 
 def _skip_if_no_scipy():
     try:
@@ -336,7 +358,6 @@ def get_locales(prefix=None, normalize=True,
         # raw_locales is "\n" seperated list of locales
         # it may contain non-decodable parts, so split
         # extract what we can and then rejoin.
-        locales = raw_locales.split(b'\n')
         raw_locales = []
         for x in raw_locales:
             try:
@@ -599,9 +620,12 @@ def assert_series_equal(left, right, check_dtype=True,
     else:
         assert_index_equal(left.index, right.index)
     if check_index_type:
-        assert_isinstance(left.index, type(right.index))
-        assert_attr_equal('dtype', left.index, right.index)
-        assert_attr_equal('inferred_type', left.index, right.index)
+        for level in range(left.index.nlevels):
+            lindex = left.index.get_level_values(level)
+            rindex = right.index.get_level_values(level)
+            assert_isinstance(lindex, type(rindex))
+            assert_attr_equal('dtype', lindex, rindex)
+            assert_attr_equal('inferred_type', lindex, rindex)
 
 # This could be refactored to use the NDFrame.equals method
 def assert_frame_equal(left, right, check_dtype=True,
@@ -648,9 +672,12 @@ def assert_frame_equal(left, right, check_dtype=True,
                                 check_exact=check_exact)
 
     if check_index_type:
-        assert_isinstance(left.index, type(right.index))
-        assert_attr_equal('dtype', left.index, right.index)
-        assert_attr_equal('inferred_type', left.index, right.index)
+        for level in range(left.index.nlevels):
+            lindex = left.index.get_level_values(level)
+            rindex = right.index.get_level_values(level)
+            assert_isinstance(lindex, type(rindex))
+            assert_attr_equal('dtype', lindex, rindex)
+            assert_attr_equal('inferred_type', lindex, rindex)
     if check_column_type:
         assert_isinstance(left.columns, type(right.columns))
         assert_attr_equal('dtype', left.columns, right.columns)
@@ -718,25 +745,30 @@ def getArangeMat():
 def makeStringIndex(k=10):
     return Index([rands(10) for _ in range(k)])
 
-
 def makeUnicodeIndex(k=10):
     return Index([randu(10) for _ in range(k)])
 
+def makeBoolIndex(k=10):
+    if k == 1:
+        return Index([True])
+    elif k == 2:
+        return Index([False,True])
+    return Index([False,True] + [False]*(k-2))
 
 def makeIntIndex(k=10):
     return Index(lrange(k))
 
-
 def makeFloatIndex(k=10):
     values = sorted(np.random.random_sample(k)) - np.random.random_sample(1)
     return Index(values * (10 ** np.random.randint(0, 9)))
 
-
 def makeDateIndex(k=10, freq='B'):
     dt = datetime(2000, 1, 1)
     dr = bdate_range(dt, periods=k, freq=freq)
     return DatetimeIndex(dr)
 
+def makeTimedeltaIndex(k=10, freq='D'):
+    return TimedeltaIndex(start='1 day',periods=k,freq=freq)
 
 def makePeriodIndex(k=10):
     dt = datetime(2000, 1, 1)
@@ -848,11 +880,12 @@ def makeCustomIndex(nentries, nlevels, prefix='#', names=False, ndupe_l=None,
        label will repeated at the corresponding level, you can specify just
        the first few, the rest will use the default ndupe_l of 1.
        len(ndupe_l) <= nlevels.
-    idx_type - "i"/"f"/"s"/"u"/"dt/"p".
+    idx_type - "i"/"f"/"s"/"u"/"dt"/"p"/"td".
        If idx_type is not None, `idx_nlevels` must be 1.
        "i"/"f" creates an integer/float index,
        "s"/"u" creates a string/unicode index
        "dt" create a datetime index.
+       "td" create a datetime index.
 
         if unspecified, string labels will be generated.
     """
@@ -863,7 +896,7 @@ def makeCustomIndex(nentries, nlevels, prefix='#', names=False, ndupe_l=None,
     assert (names is None or names is False
             or names is True or len(names) is nlevels)
     assert idx_type is None or \
-        (idx_type in ('i', 'f', 's', 'u', 'dt', 'p') and nlevels == 1)
+        (idx_type in ('i', 'f', 's', 'u', 'dt', 'p', 'td') and nlevels == 1)
 
     if names is True:
         # build default names
@@ -878,7 +911,8 @@ def makeCustomIndex(nentries, nlevels, prefix='#', names=False, ndupe_l=None,
 
     # specific 1D index type requested?
     idx_func = dict(i=makeIntIndex, f=makeFloatIndex, s=makeStringIndex,
-                    u=makeUnicodeIndex, dt=makeDateIndex, p=makePeriodIndex).get(idx_type)
+                    u=makeUnicodeIndex, dt=makeDateIndex, td=makeTimedeltaIndex,
+                    p=makePeriodIndex).get(idx_type)
     if idx_func:
         idx = idx_func(nentries)
         # but we need to fill in the name
@@ -887,7 +921,7 @@ def makeCustomIndex(nentries, nlevels, prefix='#', names=False, ndupe_l=None,
         return idx
     elif idx_type is not None:
         raise ValueError('"%s" is not a legal value for `idx_type`, use  '
-                         '"i"/"f"/"s"/"u"/"dt/"p".' % idx_type)
+                         '"i"/"f"/"s"/"u"/"dt/"p"/"td".' % idx_type)
 
     if len(ndupe_l) < nlevels:
         ndupe_l.extend([1] * (nlevels - len(ndupe_l)))
@@ -944,11 +978,12 @@ def makeCustomDataframe(nrows, ncols, c_idx_names=True, r_idx_names=True,
         doesn't divide nrows/ncol, the last label might have lower multiplicity.
    dtype - passed to the DataFrame constructor as is, in case you wish to
         have more control in conjuncion with a custom `data_gen_f`
-   r_idx_type, c_idx_type -  "i"/"f"/"s"/"u"/"dt".
+   r_idx_type, c_idx_type -  "i"/"f"/"s"/"u"/"dt"/"td".
        If idx_type is not None, `idx_nlevels` must be 1.
        "i"/"f" creates an integer/float index,
        "s"/"u" creates a string/unicode index
        "dt" create a datetime index.
+       "td" create a timedelta index.
 
         if unspecified, string labels will be generated.
 
@@ -981,9 +1016,9 @@ def makeCustomDataframe(nrows, ncols, c_idx_names=True, r_idx_names=True,
     assert c_idx_nlevels > 0
     assert r_idx_nlevels > 0
     assert r_idx_type is None or \
-        (r_idx_type in ('i', 'f', 's', 'u', 'dt', 'p') and r_idx_nlevels == 1)
+        (r_idx_type in ('i', 'f', 's', 'u', 'dt', 'p', 'td') and r_idx_nlevels == 1)
     assert c_idx_type is None or \
-        (c_idx_type in ('i', 'f', 's', 'u', 'dt', 'p') and c_idx_nlevels == 1)
+        (c_idx_type in ('i', 'f', 's', 'u', 'dt', 'p', 'td') and c_idx_nlevels == 1)
 
     columns = makeCustomIndex(ncols, nlevels=c_idx_nlevels, prefix='C',
                               names=c_idx_names, ndupe_l=c_ndupe_l,
@@ -1216,7 +1251,7 @@ _network_errno_vals = (
 # and conditionally raise on these exception types
 _network_error_classes = (IOError, httplib.HTTPException)
 
-if sys.version_info[:2] >= (3,3):
+if sys.version_info >= (3, 3):
     _network_error_classes += (TimeoutError,)
 
 def can_connect(url, error_classes=_network_error_classes):
